@@ -1,7 +1,7 @@
-# Code generation format strings for UFC (Unified Form-assembly Code) v. 1.4.0.
+# Code generation format strings for UFC (Unified Form-assembly Code) v. 1.5.0.
 # This code is released into the public domain.
 #
-# The FEniCS Project (http://www.fenicsproject.org/) 2006-2014
+# The FEniCS Project (http://www.fenicsproject.org/) 2006-2015
 
 cell_integral_combined = """\
 /// This class defines the interface for the tabulation of the cell
@@ -433,13 +433,14 @@ public:
 %(num_cells)s
   }
 
-  /// Tabulate the tensor for the contribution from custom domain
+  /// Tabulate the tensor for the contribution from a custom domain
   virtual void tabulate_tensor(double* A,
                                const double * const * w,
                                const double* vertex_coordinates,
                                std::size_t num_quadrature_points,
                                const double* quadrature_points,
                                const double* quadrature_weights,
+                               const double* facet_normals,
                                int cell_orientation) const
   {
 %(tabulate_tensor)s
@@ -470,13 +471,14 @@ public:
   /// Return the number of cells involved in evaluation of the integral
   virtual std::size_t num_cells() const;
 
-  /// Tabulate the tensor for the contribution from custom domain
+  /// Tabulate the tensor for the contribution from a custom domain
   virtual void tabulate_tensor(double* A,
                                const double * const * w,
                                const double* vertex_coordinates,
                                std::size_t num_quadrature_points,
                                const double* quadrature_points,
                                const double* quadrature_weights,
+                               const double* facet_normals,
                                int cell_orientation) const;
 
 };
@@ -507,13 +509,14 @@ std::size_t %(classname)s::num_cells() const
 %(num_cells)s
 }
 
-/// Tabulate the tensor for the contribution from custom domain
+/// Tabulate the tensor for the contribution from a custom domain
 void %(classname)s::tabulate_tensor(double* A,
                                     const double * const * w,
                                     const double* vertex_coordinates,
                                     std::size_t num_quadrature_points,
                                     const double* quadrature_points,
                                     const double* quadrature_weights,
+                                    const double* facet_normals,
                                     int cell_orientation) const
 {
 %(tabulate_tensor)s
