@@ -33,7 +33,6 @@ UFC function from an (optimized) intermediate representation (OIR).
 from ffc.log import info, begin, end, debug_code
 from ffc.cpp import format, indent
 from ffc.cpp import set_exception_handling
-#from ffc.cpp import set_float_formatting, set_exception_handling
 
 # FFC code generation modules
 from ffc.evaluatebasis import _evaluate_basis, _evaluate_basis_all
@@ -44,6 +43,7 @@ from ffc.interpolatevertexvalues import interpolate_vertex_values
 
 from ffc.representation import pick_representation
 
+
 # Errors issued for non-implemented functions
 def _not_implemented(function_name, return_null=False):
     body = format["exception"]("%s not yet implemented." % function_name)
@@ -51,6 +51,7 @@ def _not_implemented(function_name, return_null=False):
         body += "\n" + format["return"](0)
     return body
 
+
 def generate_code(ir, prefix, parameters):
     "Generate code from intermediate representation."
 
@@ -68,38 +69,45 @@ def generate_code(ir, prefix, parameters):
 
     # Generate code for elements
     info("Generating code for %d element(s)" % len(ir_elements))
-    code_elements = [_generate_element_code(ir, prefix, parameters) for ir in ir_elements]
+    code_elements = [_generate_element_code(ir, prefix, parameters)
+                     for ir in ir_elements]
 
     # Generate code for dofmaps
     info("Generating code for %d dofmap(s)" % len(ir_dofmaps))
-    code_dofmaps = [_generate_dofmap_code(ir, prefix, parameters) for ir in ir_dofmaps]
+    code_dofmaps = [_generate_dofmap_code(ir, prefix, parameters)
+                    for ir in ir_dofmaps]
 
     # Generate code for integrals
     info("Generating code for integrals")
-    code_integrals = [_generate_integral_code(ir, prefix, parameters) for ir in ir_integrals]
+    code_integrals = [_generate_integral_code(ir, prefix, parameters)
+                      for ir in ir_integrals]
 
     # Generate code for forms
     info("Generating code for forms")
-    code_forms = [_generate_form_code(ir, prefix, parameters) for ir in ir_forms]
+    code_forms = [_generate_form_code(ir, prefix, parameters)
+                  for ir in ir_forms]
 
     end()
 
     return code_elements, code_dofmaps, code_integrals, code_forms
 
+
 def _generate_element_code(ir, prefix, parameters):
     "Generate code for finite element from intermediate representation."
 
     # Skip code generation if ir is None
-    if ir is None: return None
+    if ir is None:
+        return None
 
     # Prefetch formatting to speedup code generation
-    ret        = format["return"]
-    classname  = format["classname finite_element"]
+    ret = format["return"]
+    classname = format["classname finite_element"]
     do_nothing = format["do nothing"]
-    create     = format["create foo"]
+    create = format["create foo"]
 
     # Codes generated together
-    (evaluate_dof_code, evaluate_dofs_code) = evaluate_dof_and_dofs(ir["evaluate_dof"])
+    (evaluate_dof_code, evaluate_dofs_code) \
+        = evaluate_dof_and_dofs(ir["evaluate_dof"])
 
     # Generate code
     code = {}
@@ -118,15 +126,20 @@ def _generate_element_code(ir, prefix, parameters):
     code["value_dimension"] = _value_dimension(ir["value_dimension"])
     code["evaluate_basis"] = _evaluate_basis(ir["evaluate_basis"])
     code["evaluate_basis_all"] = _evaluate_basis_all(ir["evaluate_basis"])
-    code["evaluate_basis_derivatives"] = _evaluate_basis_derivatives(ir["evaluate_basis"])
-    code["evaluate_basis_derivatives_all"] = _evaluate_basis_derivatives_all(ir["evaluate_basis"])
+    code["evaluate_basis_derivatives"] \
+        = _evaluate_basis_derivatives(ir["evaluate_basis"])
+    code["evaluate_basis_derivatives_all"] \
+        = _evaluate_basis_derivatives_all(ir["evaluate_basis"])
     code["evaluate_dof"] = evaluate_dof_code
     code["evaluate_dofs"] = evaluate_dofs_code
-    code["interpolate_vertex_values"] = interpolate_vertex_values(ir["interpolate_vertex_values"])
-    code["map_from_reference_cell"] = _not_implemented("map_from_reference_cell")
+    code["interpolate_vertex_values"] \
+        = interpolate_vertex_values(ir["interpolate_vertex_values"])
+    code["map_from_reference_cell"] \
+        = _not_implemented("map_from_reference_cell")
     code["map_to_reference_cell"] = _not_implemented("map_to_reference_cell")
     code["num_sub_elements"] = ret(ir["num_sub_elements"])
-    code["create_sub_element"] = _create_foo(prefix, "finite_element", ir["create_sub_element"])
+    code["create_sub_element"] = _create_foo(prefix, "finite_element",
+                                             ir["create_sub_element"])
     code["create"] = ret(create(code["classname"]))
 
     # Postprocess code
@@ -134,22 +147,24 @@ def _generate_element_code(ir, prefix, parameters):
 
     return code
 
+
 def _generate_dofmap_code(ir, prefix, parameters):
     "Generate code for dofmap from intermediate representation."
 
     # Skip code generation if ir is None
-    if ir is None: return None
+    if ir is None:
+        return None
 
     # Prefetch formatting to speedup code generation
-    ret        = format["return"]
-    classname  = format["classname dofmap"]
-    declare    = format["declaration"]
-    assign     = format["assign"]
+    ret = format["return"]
+    classname = format["classname dofmap"]
+    declare = format["declaration"]
+    assign = format["assign"]
     do_nothing = format["do nothing"]
-    switch     = format["switch"]
-    f_int      = format["int"]
-    f_d        = format["argument dimension"]
-    create     = format["create foo"]
+    switch = format["switch"]
+    f_int = format["int"]
+    f_d = format["argument dimension"]
+    create = format["create foo"]
 
     # Generate code
     code = {}
@@ -160,19 +175,26 @@ def _generate_dofmap_code(ir, prefix, parameters):
     code["initializer_list"] = ""
     code["destructor"] = do_nothing
     code["signature"] = ret('"%s"' % ir["signature"])
-    code["needs_mesh_entities"] = _needs_mesh_entities(ir["needs_mesh_entities"])
+    code["needs_mesh_entities"] \
+        = _needs_mesh_entities(ir["needs_mesh_entities"])
     code["topological_dimension"] = ret(ir["topological_dimension"])
     code["geometric_dimension"] = ret(ir["geometric_dimension"])
     code["global_dimension"] = _global_dimension(ir["global_dimension"])
     code["local_dimension"] = ret(ir["local_dimension"])
     code["num_facet_dofs"] = ret(ir["num_facet_dofs"])
-    code["num_entity_dofs"] = switch(f_d, [ret(num) for num in ir["num_entity_dofs"]], ret(f_int(0)))
+    code["num_entity_dofs"] \
+        = switch(f_d, [ret(num) for num in ir["num_entity_dofs"]],
+                 ret(f_int(0)))
     code["tabulate_dofs"] = _tabulate_dofs(ir["tabulate_dofs"])
-    code["tabulate_facet_dofs"] = _tabulate_facet_dofs(ir["tabulate_facet_dofs"])
-    code["tabulate_entity_dofs"] = _tabulate_entity_dofs(ir["tabulate_entity_dofs"])
-    code["tabulate_coordinates"] = _tabulate_coordinates(ir["tabulate_coordinates"])
+    code["tabulate_facet_dofs"] \
+        = _tabulate_facet_dofs(ir["tabulate_facet_dofs"])
+    code["tabulate_entity_dofs"] \
+        = _tabulate_entity_dofs(ir["tabulate_entity_dofs"])
+    code["tabulate_coordinates"] \
+        = _tabulate_coordinates(ir["tabulate_coordinates"])
     code["num_sub_dofmaps"] = ret(ir["num_sub_dofmaps"])
-    code["create_sub_dofmap"] = _create_foo(prefix, "dofmap", ir["create_sub_dofmap"])
+    code["create_sub_dofmap"] = _create_foo(prefix, "dofmap",
+                                            ir["create_sub_dofmap"])
     code["create"] = ret(create(code["classname"]))
 
     # Postprocess code
@@ -180,11 +202,13 @@ def _generate_dofmap_code(ir, prefix, parameters):
 
     return code
 
+
 def _generate_integral_code(ir, prefix, parameters):
     "Generate code for integrals from intermediate representation."
 
     # Skip code generation if ir is None
-    if ir is None: return None
+    if ir is None:
+        return None
 
     # Select representation
     r = pick_representation(ir["representation"])
@@ -193,21 +217,36 @@ def _generate_integral_code(ir, prefix, parameters):
     code = r.generate_integral_code(ir, prefix, parameters)
 
     # Indent code (unused variables should already be removed)
-    if ir["representation"] != "uflacs": # FIXME: Remove this quick hack
+    # FIXME: Remove this quick hack
+    if ir["representation"] != "uflacs":
         _indent_code(code)
 
     return code
 
+
+def _generate_original_coefficient_position(original_coefficient_positions):
+    # TODO: I don't know how to implement this using the format dict,
+    # this will do for now:
+    initializer_list = ", ".join(str(i)
+                                 for i in original_coefficient_positions)
+    code = '\n'.join([
+        "static const std::vector<std::size_t> position({%s});"
+        % initializer_list, "return position[i];",
+        ])
+    return code
+
+
 def _generate_form_code(ir, prefix, parameters):
     "Generate code for form from intermediate representation."
 
     # Skip code generation if ir is None
-    if ir is None: return None
+    if ir is None:
+        return None
 
     # Prefetch formatting to speedup code generation
-    ret =         format["return"]
-    classname =   format["classname form"]
-    do_nothing =  format["do nothing"]
+    ret = format["return"]
+    classname = format["classname form"]
+    do_nothing = format["do nothing"]
 
     # Generate code
     code = {}
@@ -220,64 +259,58 @@ def _generate_form_code(ir, prefix, parameters):
     code["destructor"] = do_nothing
 
     code["signature"] = ret('"%s"' % ir["signature"])
+    code["original_coefficient_position"] \
+        = _generate_original_coefficient_position(ir["original_coefficient_positions"])
     code["rank"] = ret(ir["rank"])
     code["num_coefficients"] = ret(ir["num_coefficients"])
 
-    code["num_cell_domains"] = ret(ir["num_cell_domains"])
-    code["num_exterior_facet_domains"] = ret(ir["num_exterior_facet_domains"])
-    code["num_interior_facet_domains"] = ret(ir["num_interior_facet_domains"])
-    code["num_point_domains"] = ret(ir["num_point_domains"])
-    code["num_custom_domains"] = ret(ir["num_custom_domains"])
-
-    code["has_cell_integrals"] = _has_foo_integrals(ir, "cell")
-    code["has_exterior_facet_integrals"] = _has_foo_integrals(ir, "exterior_facet")
-    code["has_interior_facet_integrals"] = _has_foo_integrals(ir, "interior_facet")
-    code["has_point_integrals"] = _has_foo_integrals(ir, "point")
-    code["has_custom_integrals"] = _has_foo_integrals(ir, "custom")
-
-    code["create_finite_element"] = _create_foo(prefix, "finite_element", ir["create_finite_element"])
+    code["create_finite_element"] = _create_foo(prefix, "finite_element",
+                                                ir["create_finite_element"])
     code["create_dofmap"] = _create_foo(prefix, "dofmap", ir["create_dofmap"])
 
-    code["create_cell_integral"] = _create_foo_integral(ir, "cell", prefix)
-    code["create_exterior_facet_integral"] = _create_foo_integral(ir, "exterior_facet", prefix)
-    code["create_interior_facet_integral"] = _create_foo_integral(ir, "interior_facet", prefix)
-    code["create_point_integral"] = _create_foo_integral(ir, "point", prefix)
-    code["create_custom_integral"] = _create_foo_integral(ir, "custom", prefix)
-
-    code["create_default_cell_integral"] = _create_default_foo_integral(ir, "cell", prefix)
-    code["create_default_exterior_facet_integral"] = _create_default_foo_integral(ir, "exterior_facet", prefix)
-    code["create_default_interior_facet_integral"] = _create_default_foo_integral(ir, "interior_facet", prefix)
-    code["create_default_point_integral"] = _create_default_foo_integral(ir, "point", prefix)
-    code["create_default_custom_integral"] = _create_default_foo_integral(ir, "custom", prefix)
+    integral_types = ["cell", "exterior_facet", "interior_facet", "point",
+                      "custom"]
+    for integral_type in integral_types:
+        code["num_%s_domains" % integral_type] = ret(ir["num_%s_domains"
+                                                        % integral_type])
+        code["has_%s_integrals" % integral_type] \
+            = _has_foo_integrals(ir, integral_type)
+        code["create_%s_integral" % integral_type] \
+            = _create_foo_integral(ir, integral_type, prefix)
+        code["create_default_%s_integral" % integral_type] \
+            = _create_default_foo_integral(ir, integral_type, prefix)
 
     # Postprocess code
     _postprocess_code(code, parameters)
-    #debug_code(code, "form")
 
     return code
 
 #--- Code generation for non-trivial functions ---
 
+
 def _value_dimension(ir):
     "Generate code for value_dimension."
-    ret =   format["return"]
-    axis =  format["argument axis"]
+    ret = format["return"]
+    axis = format["argument axis"]
     f_int = format["int"]
 
     if ir == ():
         return ret(1)
     return format["switch"](axis, [ret(n) for n in ir], ret(f_int(0)))
 
+
 def _needs_mesh_entities(ir):
     """
     Generate code for needs_mesh_entities. ir is a list of num dofs
     per entity.
     """
-    ret =       format["return"]
-    boolean =   format["bool"]
+    ret = format["return"]
+    boolean = format["bool"]
     dimension = format["argument dimension"]
 
-    return format["switch"](dimension, [ret(boolean(c)) for c in ir], ret(boolean(False)))
+    return format["switch"](dimension, [ret(boolean(c)) for c in ir],
+                            ret(boolean(False)))
+
 
 def _global_dimension(ir):
     """Generate code for global_dimension. ir[0] is a list of num dofs per
@@ -285,7 +318,7 @@ def _global_dimension(ir):
 
     num_dofs = ir[0]
     component = format["component"]
-    entities =  format["num entities"]
+    entities = format["num entities"]
     dimension = format["inner product"]([format["int"](d) for d in num_dofs],
                                         [component(entities, d)
                                          for d in range(len(num_dofs))])
@@ -301,36 +334,39 @@ def _global_dimension(ir):
 
     return code
 
+
 def _tabulate_facet_dofs(ir):
     "Generate code for tabulate_facet_dofs."
 
-    assign =    format["assign"]
+    assign = format["assign"]
     component = format["component"]
-    dofs =      format["argument dofs"]
+    dofs = format["argument dofs"]
     cases = ["\n".join(assign(component(dofs, i), dof)
                        for (i, dof) in enumerate(facet))
              for facet in ir]
     return format["switch"](format["facet"](None), cases)
 
+
 def _tabulate_dofs(ir):
     "Generate code for tabulate_dofs."
 
     # Prefetch formats
-    add =                 format["addition"]
-    iadd =                format["iadd"]
-    multiply =            format["multiply"]
-    assign =              format["assign"]
-    component =           format["component"]
-    entity_index =        format["entity index"]
+    add = format["addition"]
+    iadd = format["iadd"]
+    multiply = format["multiply"]
+    assign = format["assign"]
+    component = format["component"]
+    entity_index = format["entity index"]
     num_entities_format = format["num entities"]
-    unsigned_int =        format["uint declaration"]
-    dofs_variable =       format["argument dofs"]
+    unsigned_int = format["uint declaration"]
+    dofs_variable = format["argument dofs"]
 
     if ir is None:
         return assign(component(dofs_variable, 0), 0)
 
     # Extract representation
-    (dofs_per_element, num_dofs_per_element, num_entities, need_offset, fakes) = ir
+    (dofs_per_element, num_dofs_per_element, num_entities,
+     need_offset, fakes) = ir
 
     # Declare offset if needed
     code = []
@@ -355,7 +391,8 @@ def _tabulate_dofs(ir):
         for (dim, num) in enumerate(num_dofs):
 
             # Ignore if no dofs for this dimension
-            if not num[0]: continue
+            if not num[0]:
+                continue
 
             for (k, dofs) in enumerate(num):
                 v = multiply([len(num[k]), component(entity_index, (dim, k))])
@@ -373,6 +410,7 @@ def _tabulate_dofs(ir):
 
     return "\n".join(code)
 
+
 def _tabulate_coordinates(ir):
     "Generate code for tabulate_coordinates."
 
@@ -383,16 +421,15 @@ def _tabulate_coordinates(ir):
 
     # Extract formats:
     inner_product = format["inner product"]
-    component =     format["component"]
-    precision =     format["float"]
-    assign =        format["assign"]
-    f_x =           format["vertex_coordinates"]
-    coordinates =   format["argument coordinates"]
+    component = format["component"]
+    precision = format["float"]
+    assign = format["assign"]
+    f_x = format["vertex_coordinates"]
+    coordinates = format["argument coordinates"]
 
     # Extract coordinates and cell dimension
     gdim = ir["gdim"]
     tdim = ir["tdim"]
-    #cell_dim = len(ir[0])
 
     # Aid mapping points from reference to physical element
     coefficients = affine_weights(tdim)
@@ -408,10 +445,11 @@ def _tabulate_coordinates(ir):
             value = inner_product(w, coords)
 
             # Assign coordinate
-            code.append(assign(component(coordinates, (i, j)), value))
+            code.append(assign(component(coordinates, (i*gdim + j)), value))
 
     return "\n".join(code)
 
+
 def _tabulate_entity_dofs(ir):
     "Generate code for tabulate_entity_dofs."
 
@@ -419,15 +457,16 @@ def _tabulate_entity_dofs(ir):
     entity_dofs, num_dofs_per_entity = ir
 
     # Prefetch formats
-    assign =    format["assign"]
+    assign = format["assign"]
     component = format["component"]
-    f_d =       format["argument dimension"]
-    f_i =       format["argument entity"]
-    dofs =      format["argument dofs"]
+    f_d = format["argument dimension"]
+    f_i = format["argument entity"]
+    dofs = format["argument dofs"]
 
     # Add check that dimension and number of mesh entities is valid
     dim = len(num_dofs_per_entity)
-    excpt = format["exception"]("%s is larger than dimension (%d)" % (f_d, dim - 1))
+    excpt = format["exception"]("%s is larger than dimension (%d)"
+                                % (f_d, dim - 1))
     code = [format["if"]("%s > %d" % (f_d, dim-1), excpt)]
 
     # Generate cases for each dimension:
@@ -435,7 +474,8 @@ def _tabulate_entity_dofs(ir):
     for d in range(dim):
 
         # Ignore if no entities for this dimension
-        if num_dofs_per_entity[d] == 0: continue
+        if num_dofs_per_entity[d] == 0:
+            continue
 
         # Add check that given entity is valid:
         num_entities = len(entity_dofs[d].keys())
@@ -456,31 +496,37 @@ def _tabulate_entity_dofs(ir):
 
     return "\n".join(code)
 
+
 #--- Utility functions ---
 
+
 def _create_foo(prefix, class_name, postfix, numbers=None):
     "Generate code for create_<foo>."
-    f_i    = format["argument sub"]
-    ret    = format["return"]
+    f_i = format["argument sub"]
+    ret = format["return"]
     create = format["create foo"]
 
-    class_names = ["%s_%s_%d" % (prefix.lower(), class_name, i) for i in postfix]
+    class_names = ["%s_%s_%d" % (prefix.lower(), class_name, i)
+                   for i in postfix]
     cases = [ret(create(name)) for name in class_names]
     default = ret(0)
     return format["switch"](f_i, cases, default=default, numbers=numbers)
 
+
 def _create_foo_integral(ir, integral_type, prefix):
     "Generate code for create_<foo>_integral."
     class_name = integral_type + "_integral_" + str(ir["id"])
     postfix = ir["create_" + integral_type + "_integral"]
     return _create_foo(prefix, class_name, postfix, numbers=postfix)
 
+
 def _has_foo_integrals(ir, integral_type):
     ret = format["return"]
     b = format["bool"]
     i = ir["has_%s_integrals" % integral_type]
     return ret(b(i))
 
+
 def _create_default_foo_integral(ir, integral_type, prefix):
     "Generate code for create_default_<foo>_integral."
     ret = format["return"]
@@ -493,21 +539,27 @@ def _create_default_foo_integral(ir, integral_type, prefix):
         name = "%s_%s_%s" % (prefix.lower(), class_name, postfix)
         return ret(create(name))
 
+
 def _postprocess_code(code, parameters):
     "Postprocess generated code."
     _indent_code(code)
     _remove_code(code, parameters)
 
+
 def _indent_code(code):
     "Indent code that should be indented."
     for key in code:
-        if not key in ("classname", "members", "constructor_arguments", "initializer_list", "additional_includes_set", "restrict"):
+        if not key in ("classname", "members", "constructor_arguments",
+                       "initializer_list", "additional_includes_set",
+                       "restrict"):
             code[key] = indent(code[key], 4)
 
+
 def _remove_code(code, parameters):
     "Remove code that should not be generated."
     for key in code:
         flag = "no-" + key
         if flag in parameters and parameters[flag]:
-            msg = "// Function %s not generated (compiled with -f%s)" % (key, flag)
+            msg = "// Function %s not generated (compiled with -f%s)" \
+                  % (key, flag)
             code[key] = format["exception"](msg)
