@@ -22,13 +22,14 @@
 # Modified by Martin Alnaes 2013
 #
 # First added:  2009-12-16
-# Last changed: 2014-04-15
+# Last changed: 2014-06-09
 
 # Python modules
 import re, numpy, platform
 
 # FFC modules
 from ffc.log import debug, error
+from six.moves import zip
 
 # Mapping of restrictions
 _fixed_map = {None: "", "+": "_0", "-": "_1"}
@@ -117,6 +118,8 @@ format.update({
     "bessel_k":       lambda v, n: "boost::math::cyl_bessel_k(%s, %s)" % (str(n), str(v)),
     "bessel_y":       lambda v, n: "boost::math::cyl_neumann(%s, %s)" % (str(n), str(v)),
     "absolute value": lambda v: "std::abs(%s)" % str(v),
+    "min value":      lambda l, r: "std::min(%s, %s)" % (str(l), str(r)),
+    "max value":      lambda l, r: "std::max(%s, %s)" % (str(l), str(r)),
     "sqrt":           lambda v: "std::sqrt(%s)" % str(v),
     "addition":       lambda v: _add(v),
     "multiply":       lambda v: _multiply(v),
@@ -216,25 +219,26 @@ format.update({
     "call basis_derivatives_all": lambda i, s: "_evaluate_basis_derivatives_all(n, %s, x, vertex_coordinates, cell_orientation);" % s,
 
     # quadrature code generators
-    "integration points": "ip",
-    "first free index":   "j",
-    "second free index":  "k",
-    "geometry constant":  lambda i: "G[%d]" % i,
-    "ip constant":        lambda i: "I[%d]" % i,
-    "basis constant":     lambda i: "B[%d]" % i,
-    "conditional":        lambda i: "C[%d]" % i,
-    "evaluate conditional":lambda i,j,k: "(%s) ? %s : %s" % (i,j,k),
-#    "geometry constant":  lambda i: "G%d" % i,
-#    "ip constant":        lambda i: "I%d" % i,
-#    "basis constant":     lambda i: "B%d" % i,
-    "function value":     lambda i: "F%d" % i,
-    "nonzero columns":    lambda i: "nzc%d" % i,
-    "weight":             lambda i: "W" if i is None else "W%d" % (i),
-    "psi name":           lambda c, et, e, co, d, a: _generate_psi_name(c, et, e, co, d, a),
+    "integration points":   "ip",
+    "first free index":     "j",
+    "second free index":    "k",
+    "geometry constant":    lambda i: "G[%d]" % i,
+    "ip constant":          lambda i: "I[%d]" % i,
+    "basis constant":       lambda i: "B[%d]" % i,
+    "conditional":          lambda i: "C[%d]" % i,
+    "evaluate conditional": lambda i,j,k: "(%s) ? %s : %s" % (i,j,k),
+#    "geometry constant":   lambda i: "G%d" % i,
+#    "ip constant":         lambda i: "I%d" % i,
+#    "basis constant":      lambda i: "B%d" % i,
+    "function value":       lambda i: "F%d" % i,
+    "nonzero columns":      lambda i: "nzc%d" % i,
+    "weight":               lambda i: "W" if i is None else "W%d" % (i),
+    "psi name":             lambda c, et, e, co, d, a: _generate_psi_name(c, et, e, co, d, a),
     # both
-    "free indices":       ["r","s","t","u"],
-    "matrix index":       lambda i, j, range_j: _matrix_index(i, str(j), str(range_j)),
-    "quadrature point":   lambda i, gdim: "quadrature_points + %s*%d" % (i, gdim)
+    "free indices":         ["r","s","t","u"],
+    "matrix index":         lambda i, j, range_j: _matrix_index(i, str(j), str(range_j)),
+    "quadrature point":     lambda i, gdim: "quadrature_points + %s*%d" % (i, gdim),
+    "facet_normal_custom":  lambda gdim: _generate_facet_normal_custom(gdim),
 })
 
 # Misc
@@ -249,7 +253,7 @@ format.update({
 })
 
 # Code snippets
-from codesnippets import *
+from .codesnippets import *
 
 format.update({
     "compute_jacobian":         lambda tdim, gdim, r=None: \
@@ -279,9 +283,11 @@ format.update({
     "header_c":                 header_c,
     "footer":                   footer,
     "eval_basis_decl":          eval_basis_decl,
+    "eval_basis_init":          eval_basis_init,
     "eval_basis":               eval_basis,
     "eval_basis_copy":          eval_basis_copy,
     "eval_derivs_decl":         eval_derivs_decl,
+    "eval_derivs_init":         eval_derivs_init,
     "eval_derivs":              eval_derivs,
     "eval_derivs_copy":         eval_derivs_copy,
     "extract_cell_coordinates": lambda offset, r : "const double* vertex_coordinates_%d = vertex_coordinates + %d;" % (r, offset)
@@ -453,7 +459,7 @@ def _generate_switch(variable, cases, default=None, numbers=None):
 
     # Create numbers for switch
     if numbers is None:
-        numbers = range(len(cases))
+        numbers = list(range(len(cases)))
 
     # Create switch
     code = "switch (%s)\n{\n" % variable
@@ -592,7 +598,7 @@ def _generate_normal(tdim, gdim, integral_type, reference_normal=False):
     # Choose snippets
     direction = normal_direction[tdim][gdim]
 
-    assert (facet_normal[tdim].has_key(gdim)),\
+    assert (gdim in facet_normal[tdim]),\
         "Facet normal not yet implemented for this tdim/gdim combo"
     normal = facet_normal[tdim][gdim]
 
@@ -608,6 +614,14 @@ def _generate_normal(tdim, gdim, integral_type, reference_normal=False):
         error("Unsupported integral_type: %s" % str(integral_type))
     return code
 
+def _generate_facet_normal_custom(gdim):
+    "Generate code for setting facet normal in custom integrals"
+    code = format["comment"]("Set facet normal components for current quadrature point\n")
+    for i in range(gdim):
+        code += "const double n_0%d =   facet_normals[%d*ip + %d];\n" % (i, gdim, i)
+        code += "const double n_1%d = - facet_normals[%d*ip + %d];\n" % (i, gdim, i)
+    return code
+
 def _generate_cell_volume(tdim, gdim, integral_type, r=None):
     "Generate code for computing cell volume."
 
@@ -658,7 +672,7 @@ def count_ops(code):
     "Count the number of operations in code (multiply-add pairs)."
     num_add = code.count(" + ") + code.count(" - ")
     num_multiply = code.count("*") + code.count("/")
-    return (num_add + num_multiply) / 2
+    return (num_add + num_multiply) // 2
 
 def set_float_formatting(precision):
     "Set floating point formatting based on precision."
