@@ -18,41 +18,26 @@
 # Modified by Garth N. Wells, 2009.
 # Modified by Marie Rognes, 2009-2013.
 # Modified by Martin Alnaes, 2013
-#
-# First added:  2009-03-06
-# Last changed: 2013-01-25
 
 # Python modules
 from numpy import array
+import six
 
 # UFL and FIAT modules
 import ufl
+from ufl.utils.sorting import sorted_by_key
 import FIAT
 
 # FFC modules
 from ffc.log import debug, error, ffc_assert
 from ffc.quadratureelement import QuadratureElement as FFCQuadratureElement
-from ffc.timeelements import LobattoElement as FFCLobattoElement
-from ffc.timeelements import RadauElement as FFCRadauElement
 
 from ffc.mixedelement import MixedElement
 from ffc.restrictedelement import RestrictedElement
 from ffc.enrichedelement import EnrichedElement, SpaceOfReals
 
 # Dictionary mapping from cellname to dimension
-from ufl.geometry import cellname2dim
-
-# Number of entities associated with each cell name
-cellname_to_num_entities = {
-    "cell1D": None,
-    "cell2D": None,
-    "cell3D": None,
-    "interval": (2, 1),
-    "triangle": (3, 3, 1),
-    "tetrahedron": (4, 6, 4, 1),
-    "quadrilateral": (4, 4, 1),
-    "hexahedron": (8, 12, 6, 1),
-    }
+from ufl.cell import cellname2dim
 
 # Element families supported by FFC
 supported_families = ("Brezzi-Douglas-Marini",
@@ -70,12 +55,6 @@ supported_families = ("Brezzi-Douglas-Marini",
                       "Bubble",
                       "Quadrature")
 
-# Mapping from dimension to number of mesh sub-entities. (In principle,
-# cellname_to_num_entities contains the same information, but with string keys.)
-# DISABLED ON PURPOSE: It's better to use cell name instead of dimension
-#                      to stay generic w.r.t. future box elements.
-#entities_per_dim = {1: [2, 1], 2: [3, 3, 1], 3: [4, 6, 4, 1]}
-
 # Cache for computed elements
 _cache = {}
 
@@ -147,9 +126,11 @@ def _create_fiat_element(ufl_element):
 
     # Handle the specialized time elements
     elif family == "Lobatto" :
+        from ffc.timeelements import LobattoElement as FFCLobattoElement
         element = FFCLobattoElement(ufl_element.degree())
 
     elif family == "Radau" :
+        from ffc.timeelements import RadauElement as FFCRadauElement
         element = FFCRadauElement(ufl_element.degree())
 
     # FIXME: AL: Should this really be here?
@@ -309,7 +290,7 @@ def _indices(element, restriction_domain, dim=0):
         indices = []
         for dim in range(restriction_domain.topological_dimension() + 1):
             entities = entity_dofs[dim]
-            for (entity, index) in entities.iteritems():
+            for (entity, index) in sorted_by_key(entities):
                 indices += index
         return indices
 
