@@ -16,18 +16,16 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Martin Alnaes, 2013
-#
-# First added:  2008-09-04
-# Last changed: 2014-02-20
 
 # Python modules.
 from hashlib import sha1
 
 # UFL modules.
 import ufl
+from ufl.utils.sorting import canonicalize_metadata
 
 # FFC modules.
-from constants import FFC_VERSION
+from .constants import FFC_VERSION
 
 # UFC modules.
 from ffc.backends import ufc
@@ -35,8 +33,8 @@ from ffc.backends import ufc
 # Compute signature of all ufc headers combined
 ufc_signature = sha1(''.join(getattr(ufc, header)
                              for header in
-                             (k for k in vars(ufc).keys()
-                              if k.endswith("_header")))
+                             (k for k in sorted(vars(ufc).keys())
+                              if k.endswith("_header"))).encode('utf-8')
                               ).hexdigest()
 
 class JITObject:
@@ -58,14 +56,10 @@ class JITObject:
 
     def __hash__(self):
         "Return unique integer for form + parameters"
-
         # Check if we have computed the hash before
-        if not self._hash is None:
-            return self._hash
-
-        # Compute hash based on signature
-        self._hash = int(self.signature(), 16)
-
+        if self._hash is None:
+            # Compute hash based on signature
+            self._hash = int(self.signature(), 16)
         return self._hash
 
     def __eq__(self, other):
@@ -79,28 +73,26 @@ class JITObject:
         if not self._signature is None:
             return self._signature
 
-        # Get signature from assumed precomputed form_data
-        form_signature = self.form.form_data().signature
+        # Get signature from form
+        form_signature = self.form.signature()
 
         # Compute other relevant signatures
         parameters_signature = _parameters_signature(self.parameters)
         ffc_signature = str(FFC_VERSION)
-        #cell_signature = str(self.form.form_data().cell)
 
         # Build common signature
         signatures = [form_signature,
                       parameters_signature,
                       ffc_signature,
-                      #cell_signature,
                       ufc_signature]
         string = ";".join(signatures)
-        self._signature = sha1(string).hexdigest()
+
+        self._signature = sha1(string.encode('utf-8')).hexdigest()
 
         # Uncomment for debugging
         #print "form_signature       =", form_signature
         #print "parameters_signature =", parameters_signature
         #print "ffc_signature        =", ffc_signature
-        #print "cell_signature       =", cell_signature
         #print "signature            =", self._signature
 
         return self._signature
@@ -112,4 +104,4 @@ def _parameters_signature(parameters):
     for ignore in ignores:
         if ignore in parameters:
             del parameters[ignore]
-    return str(parameters)
+    return str(canonicalize_metadata(parameters))
