@@ -18,10 +18,7 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg 2009, 2014
-# Modified by Martin Alnaes 2013
-#
-# First added:  2009-01-07
-# Last changed: 2014-04-23
+# Modified by Martin Alnaes 2013-2014
 
 # Python modules
 import numpy, itertools, collections
@@ -31,9 +28,9 @@ from ufl.classes import Form, Integral
 from ufl.sorting import sorted_expr_sum
 
 # FFC modules
-from ffc.log import ffc_assert, info, error
+from ffc.log import ffc_assert, info, error, warning
+from ffc.utils import product
 from ffc.fiatinterface import create_element
-from ffc.fiatinterface import cellname_to_num_entities
 
 from ffc.representationutils import initialize_integral_ir
 from ffc.quadrature.tabulate_basis import tabulate_basis
@@ -41,10 +38,12 @@ from ffc.quadrature.parameters import parse_optimise_parameters
 
 from ffc.quadrature.quadraturetransformer import QuadratureTransformer
 from ffc.quadrature.optimisedquadraturetransformer import QuadratureTransformerOpt
+import six
 
 def compute_integral_ir(itg_data,
                         form_data,
                         form_id,
+                        element_numbers,
                         parameters):
     "Compute intermediate represention of integral."
 
@@ -82,16 +81,16 @@ def compute_integral_ir(itg_data,
     # Create transformer
     transformer = QuadratureTransformerClass(psi_tables,
                                              quadrature_rules,
-                                             form_data.geometric_dimension,
+                                             itg_data.domain.geometric_dimension(),
                                              itg_data.domain.topological_dimension(),
                                              ir["entitytype"],
                                              form_data.function_replace_map,
                                              ir["optimise_parameters"])
 
     # Transform integrals.
-    cellname = itg_data.domain.cell().cellname()
+    cell = itg_data.domain.cell()
     ir["trans_integrals"] = _transform_integrals_by_type(ir, transformer, integrals_dict,
-                                                         itg_data.integral_type, cellname)
+                                                         itg_data.integral_type, cell)
 
     # Save tables populated by transformer
     ir["name_map"] = transformer.name_map
@@ -106,7 +105,7 @@ def compute_integral_ir(itg_data,
 
     # Extract element data for psi_tables, needed for runtime quadrature.
     # This is used by integral type custom_integral.
-    ir["element_data"] = _extract_element_data(transformer.element_map)
+    ir["element_data"] = _extract_element_data(transformer.element_map, element_numbers)
 
     return ir
 
@@ -144,15 +143,15 @@ def sort_integrals(integrals, default_quadrature_degree, default_quadrature_rule
 
     # Create integrals from accumulated integrands.
     sorted_integrals = {}
-    for key, integrands in sorted_integrands.items():
+    for key, integrands in list(sorted_integrands.items()):
         # Summing integrands in a canonical ordering defined by UFL
         integrand = sorted_expr_sum(integrands)
         sorted_integrals[key] = Integral(integrand, integral_type, domain, subdomain_id, {}, None)
     return sorted_integrals
 
-def _transform_integrals_by_type(ir, transformer, integrals_dict, integral_type, cellname):
-    num_facets = cellname_to_num_entities[cellname][-2]
-    num_vertices = cellname_to_num_entities[cellname][0]
+def _transform_integrals_by_type(ir, transformer, integrals_dict, integral_type, cell):
+    num_facets = cell.num_facets()
+    num_vertices = cell.num_vertices()
 
     if integral_type == "cell":
         # Compute transformed integrals.
@@ -199,30 +198,39 @@ def _transform_integrals_by_type(ir, transformer, integrals_dict, integral_type,
 def _transform_integrals(transformer, integrals, integral_type):
     "Transform integrals from UFL expression to quadrature representation."
     transformed_integrals = []
-    for point, integral in integrals.items():
+    for point, integral in sorted(integrals.items()):
         transformer.update_points(point)
         terms = transformer.generate_terms(integral.integrand(), integral_type)
         transformed_integrals.append((point, terms, transformer.function_data,
                                       {}, transformer.coordinate, transformer.conditionals))
     return transformed_integrals
 
-def _extract_element_data(element_map):
+def _extract_element_data(element_map, element_numbers):
     "Extract element data for psi_tables"
 
     # Iterate over map
     element_data = {}
-    for elements in element_map.itervalues():
-        for ufl_element, counter in elements.iteritems():
+    for elements in six.itervalues(element_map):
+        for ufl_element, counter in six.iteritems(elements):
 
             # Create corresponding FIAT element
             fiat_element = create_element(ufl_element)
 
             # Compute value size
             shape = ufl_element.value_shape()
-            value_size = 1 if shape == () else itertools.product(shape)
+            value_size = 1 if shape == () else product(shape)
+
+            # Get element number
+            if ufl_element in element_numbers:
+                element_number = element_numbers[ufl_element]
+            else:
+                # FIXME: Should not be necessary, we should always know the element number
+                #warning("Missing element number, likely because vector elements are not yet supported in custom integrals.")
+                element_number = None
 
             # Store data
             element_data[counter] = {"value_size":      value_size,
-                                     "local_dimension": fiat_element.space_dimension()}
+                                     "local_dimension": fiat_element.space_dimension(),
+                                     "element_number":  element_number}
 
     return element_data
