@@ -19,15 +19,19 @@
 #
 # Modified by Peter Brune 2009
 # Modified by Anders Logg 2009, 2013
-#
-# First added:  2009-02-09
-# Last changed: 2014-04-23
 
 # Python modules.
 from numpy import shape
 
+from six import iteritems, iterkeys
+from six.moves import xrange as range
+from six import advance_iterator as next
+def firstkey(d):
+    return next(iterkeys(d))
+
 # UFL common.
 from ufl.common import product, StackDict, Stack
+from ufl.utils.sorting import sorted_by_key
 
 # UFL Classes.
 from ufl.classes import FixedIndex
@@ -44,10 +48,10 @@ from ffc.log import info, debug, error, ffc_assert
 from ffc.cpp import format
 
 # Utility and optimisation functions for quadraturegenerator.
-from quadraturetransformerbase import QuadratureTransformerBase
-from quadratureutils import create_permutations
-from reduce_operations import operation_count
-from symbolics import IP
+from .quadraturetransformerbase import QuadratureTransformerBase
+from .quadratureutils import create_permutations
+from .reduce_operations import operation_count
+from .symbolics import IP
 
 class QuadratureTransformer(QuadratureTransformerBase):
     "Transform UFL representation to quadrature code."
@@ -77,14 +81,14 @@ class QuadratureTransformer(QuadratureTransformerBase):
         for op in operands:
             # If entries does already exist we can add the code, otherwise just
             # dump them in the element tensor.
-            for key, val in op.items():
+            for key, val in sorted_by_key(op):
                 if key in code:
                     code[key].append(val)
                 else:
                     code[key] = [val]
 
         # Add sums and group if necessary.
-        for key, val in code.items():
+        for key, val in sorted_by_key(code):
 
             # Exclude all zero valued terms from sum
             value = [v for v in val if not v is None]
@@ -102,7 +106,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
                 # Add a product for each term that has duplicate code
                 expressions = []
-                for expr, num_occur in duplications.items():
+                for expr, num_occur in sorted_by_key(duplications):
                     if num_occur > 1:
                         # Pre-multiply expression with number of occurrences
                         expressions.append(f_mult([f_float(num_occur), expr]))
@@ -139,9 +143,9 @@ class QuadratureTransformer(QuadratureTransformerBase):
             # If we get an empty dict, something was zero and so is the product.
             if not op:
                 return {}
-            if len(op) > 1 or (op and op.keys()[0] != ()):
+            if len(op) > 1 or (op and firstkey(op) != ()):
                 permute.append(op)
-            elif op and op.keys()[0] == ():
+            elif op and firstkey(op) == ():
                 not_permute.append(op[()])
 
         # Create permutations.
@@ -153,10 +157,9 @@ class QuadratureTransformer(QuadratureTransformerBase):
         # Create code.
         code ={}
         if permutations:
-            for key, val in permutations.items():
+            for key, val in sorted(permutations.items()):
                 # Sort key in order to create a unique key.
-                l = list(key)
-                l.sort()
+                l = sorted(key)
 
                 # Loop products, don't multiply by '1' and if we encounter a None the product is zero.
                 # TODO: Need to find a way to remove and J_inv00 terms that might
@@ -170,7 +173,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
                         zero = True
                         break
                     elif not v:
-                        print "v: '%s'" % repr(v)
+                        print("v: '%s'" % repr(v))
                         error("should not happen")
                     elif v == "1":
                         pass
@@ -193,7 +196,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
                     code[()] = None
                     return code
                 elif not v:
-                    print "v: '%s'" % repr(v)
+                    print("v: '%s'" % repr(v))
                     error("should not happen")
                 elif v == "1":
                     pass
@@ -245,7 +248,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         #print("\n\nVisiting Power: " + repr(o))
 
         # Get base and exponent.
-        base, expo = o.operands()
+        base, expo = o.ufl_operands
 
         # Visit base to get base code.
         base_code = self.visit(base)
@@ -280,6 +283,14 @@ class QuadratureTransformer(QuadratureTransformerBase):
         # Take absolute value of operand.
         return {():f_abs(operands[0][()])}
 
+    def min_value(self, o, *operands):
+        f_min = format["min value"]
+        return {():f_min(operands[0][()], operands[1][()])}
+
+    def max_value(self, o, *operands):
+        f_max = format["max value"]
+        return {():f_max(operands[0][()], operands[1][()])}
+
     # -------------------------------------------------------------------------
     # Condition, Conditional (conditional.py).
     # -------------------------------------------------------------------------
@@ -288,7 +299,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
         # Get condition expression and do safety checks.
         # Might be a bit too strict?
         cond, = operands
-        ffc_assert(len(cond) == 1 and cond.keys()[0] == (),\
+        ffc_assert(len(cond) == 1 and firstkey(cond) == (),\
             "Condition for NotCondition should only be one function: " + repr(cond))
         return {(): format["not"](cond[()])}
 
@@ -297,9 +308,9 @@ class QuadratureTransformer(QuadratureTransformerBase):
         # Get LHS and RHS expressions and do safety checks.
         # Might be a bit too strict?
         lhs, rhs = operands
-        ffc_assert(len(lhs) == 1 and lhs.keys()[0] == (),\
+        ffc_assert(len(lhs) == 1 and firstkey(lhs) == (),\
             "LHS of Condition should only be one function: " + repr(lhs))
-        ffc_assert(len(rhs) == 1 and rhs.keys()[0] == (),\
+        ffc_assert(len(rhs) == 1 and firstkey(rhs) == (),\
             "RHS of Condition should only be one function: " + repr(rhs))
 
         # Map names from UFL to cpp.py.
@@ -322,11 +333,11 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         # Get condition and return values; and do safety check.
         cond, true, false = operands
-        ffc_assert(len(cond) == 1 and cond.keys()[0] == (),\
+        ffc_assert(len(cond) == 1 and firstkey(cond) == (),\
             "Condtion should only be one function: " + repr(cond))
-        ffc_assert(len(true) == 1 and true.keys()[0] == (),\
+        ffc_assert(len(true) == 1 and firstkey(true) == (),\
             "True value of Condtional should only be one function: " + repr(true))
-        ffc_assert(len(false) == 1 and false.keys()[0] == (),\
+        ffc_assert(len(false) == 1 and firstkey(false) == (),\
             "False value of Condtional should only be one function: " + repr(false))
 
         # Get values and test for None
@@ -542,7 +553,7 @@ class QuadratureTransformer(QuadratureTransformerBase):
                         code[mapping].append(self.__apply_transform(basis, derivatives, multi, tdim, gdim))
 
         # Add sums and group if necessary.
-        for key, val in code.items():
+        for key, val in list(code.items()):
             if len(val) > 1:
                 code[key] = f_group(f_add(val))
             elif val:
@@ -631,7 +642,15 @@ class QuadratureTransformer(QuadratureTransformerBase):
 
         # Add transformation if needed.
         transforms = []
-        if not self.integral_type == "custom":
+        if self.integral_type == "custom":
+            for i, direction in enumerate(derivatives):
+                # Custom integrals to not need transforms, so in place
+                # of the transform, we insert an identity matrix
+                ref = multi[i]
+                if ref != direction:
+                    transforms.append(0)
+
+        else:
             for i, direction in enumerate(derivatives):
                 ref = multi[i]
                 t = f_transform("JINV", ref, direction, tdim, gdim, self.restriction)
@@ -674,8 +693,8 @@ class QuadratureTransformer(QuadratureTransformerBase):
         return new_operand
 
     def _atan_2_function(self, operands, format_function):
-        x1,x2 = operands
-        x1,x2 = x1.values()[0],x2.values()[0]
+        x1, x2 = operands
+        x1, x2 = sorted(x1.values())[0], sorted(x2.values())[0]
 
         if x1 is None:
             x1 = format["floating point"](0.0)
