@@ -22,10 +22,10 @@ transformers to translate UFL expressions."""
 # Modified by Garth N. Wells, 2013
 #
 # First added:  2009-10-13
-# Last changed: 2014-04-23
+# Last changed: 2014-10-08
 
 # Python modules.
-from itertools import izip
+from six.moves import zip
 from numpy import shape, array
 
 # UFL Classes.
@@ -47,8 +47,8 @@ from ffc.tensor.multiindex import MultiIndex as FFCMultiIndex
 from ffc.representationutils import transform_component
 
 # Utility and optimisation functions for quadraturegenerator.
-from quadratureutils import create_psi_tables
-from symbolics import BASIS, IP, GEO, CONST
+from ffc.quadrature.quadratureutils import create_psi_tables
+from ffc.quadrature.symbolics import BASIS, IP, GEO, CONST
 
 class QuadratureTransformerBase(Transformer):
     "Transform UFL representation to quadrature code."
@@ -143,13 +143,13 @@ class QuadratureTransformerBase(Transformer):
         self.function_cache = {}
 
     def disp(self):
-        print "\n\n **** Displaying QuadratureTransformer ****"
-        print "\nQuadratureTransformer, element_map:\n", self.element_map
-        print "\nQuadratureTransformer, name_map:\n", self.name_map
-        print "\nQuadratureTransformer, unique_tables:\n", self.unique_tables
-        print "\nQuadratureTransformer, used_psi_tables:\n", self.used_psi_tables
-        print "\nQuadratureTransformer, psi_tables_map:\n", self.psi_tables_map
-        print "\nQuadratureTransformer, used_weights:\n", self.used_weights
+        print("\n\n **** Displaying QuadratureTransformer ****")
+        print("\nQuadratureTransformer, element_map:\n", self.element_map)
+        print("\nQuadratureTransformer, name_map:\n", self.name_map)
+        print("\nQuadratureTransformer, unique_tables:\n", self.unique_tables)
+        print("\nQuadratureTransformer, used_psi_tables:\n", self.used_psi_tables)
+        print("\nQuadratureTransformer, psi_tables_map:\n", self.psi_tables_map)
+        print("\nQuadratureTransformer, used_weights:\n", self.used_weights)
 
     def component(self):
         "Return current component tuple."
@@ -168,12 +168,12 @@ class QuadratureTransformerBase(Transformer):
     # -------------------------------------------------------------------------
     # Nothing in expr.py is handled. Can only handle children of these clases.
     def expr(self, o):
-        print "\n\nVisiting basic Expr:", repr(o), "with operands:"
+        print("\n\nVisiting basic Expr:", repr(o), "with operands:")
         error("This expression is not handled: " + repr(o))
 
     # Nothing in terminal.py is handled. Can only handle children of these clases.
     def terminal(self, o):
-        print "\n\nVisiting basic Terminal:", repr(o), "with operands:"
+        print("\n\nVisiting basic Terminal:", repr(o), "with operands:")
         error("This terminal is not handled: " + repr(o))
 
     # -------------------------------------------------------------------------
@@ -181,20 +181,16 @@ class QuadratureTransformerBase(Transformer):
     # algebra.py, differentiation.py, finiteelement.py,
     # form.py, geometry.py, indexing.py, integral.py, tensoralgebra.py, variable.py.
     # -------------------------------------------------------------------------
-    def algebra_operator(self, o, *operands):
-        print "\n\nVisiting AlgebraOperator: ", repr(o)
-        error("This type of AlgebraOperator should have been expanded!!" + repr(o))
-
     def derivative(self, o, *operands):
-        print "\n\nVisiting Derivative: ", repr(o)
+        print("\n\nVisiting Derivative: ", repr(o))
         error("All derivatives apart from Grad should have been expanded!!")
 
     def compound_tensor_operator(self, o):
-        print "\n\nVisiting CompoundTensorOperator: ", repr(o)
+        print("\n\nVisiting CompoundTensorOperator: ", repr(o))
         error("CompoundTensorOperator should have been expanded.")
 
     def label(self, o):
-        print "\n\nVisiting Label: ", repr(o)
+        print("\n\nVisiting Label: ", repr(o))
         error("What is a Lable doing in the integrand?")
 
     # -------------------------------------------------------------------------
@@ -203,39 +199,31 @@ class QuadratureTransformerBase(Transformer):
     # mathfunctions.py, restriction.py
     # -------------------------------------------------------------------------
     def condition(self, o):
-        print "\n\nVisiting Condition:", repr(o)
+        print("\n\nVisiting Condition:", repr(o))
         error("This type of Condition is not supported (yet).")
 
     def constant_value(self, o):
-        print "\n\nVisiting ConstantValue:", repr(o)
+        print("\n\nVisiting ConstantValue:", repr(o))
         error("This type of ConstantValue is not supported (yet).")
 
-    def index_annotated(self, o):
-        print "\n\nVisiting IndexAnnotated:", repr(o)
-        error("Only child classes of IndexAnnotated is supported.")
-
-    def constant_base(self, o):
-        print "\n\nVisiting ConstantBase:", repr(o)
-        error("This type of ConstantBase is not supported (yet).")
-
     def geometric_quantity(self, o):
-        print "\n\nVisiting GeometricQuantity:", repr(o)
+        print("\n\nVisiting GeometricQuantity:", repr(o))
         error("This type of GeometricQuantity is not supported (yet).")
 
     def math_function(self, o):
-        print "\n\nVisiting MathFunction:", repr(o)
+        print("\n\nVisiting MathFunction:", repr(o))
         error("This MathFunction is not supported (yet).")
 
     def atan_2_function(self, o):
-        print "\n\nVisiting Atan2Function:", repr(o)
+        print("\n\nVisiting Atan2Function:", repr(o))
         error("Atan2Function is not implemented (yet).")
 
     def bessel_function(self, o):
-        print "\n\nVisiting BesselFunction:", repr(o)
+        print("\n\nVisiting BesselFunction:", repr(o))
         error("BesselFunction is not implemented (yet).")
 
     def restricted(self, o):
-        print "\n\nVisiting Restricted:", repr(o)
+        print("\n\nVisiting Restricted:", repr(o))
         error("This type of Restricted is not supported (only positive and negative are currently supported).")
 
     # -------------------------------------------------------------------------
@@ -245,23 +233,23 @@ class QuadratureTransformerBase(Transformer):
     # AlgebraOperators (algebra.py).
     # -------------------------------------------------------------------------
     def sum(self, o, *operands):
-        print "\n\nVisiting Sum: ", repr(o)
+        print("\n\nVisiting Sum: ", repr(o))
         error("This object should be implemented by the child class.")
 
     def product(self, o, *operands):
-        print "\n\nVisiting Product: ", repr(o)
+        print("\n\nVisiting Product: ", repr(o))
         error("This object should be implemented by the child class.")
 
     def division(self, o, *operands):
-        print "\n\nVisiting Division: ", repr(o)
+        print("\n\nVisiting Division: ", repr(o))
         error("This object should be implemented by the child class.")
 
     def power(self, o):
-        print "\n\nVisiting Power: ", repr(o)
+        print("\n\nVisiting Power: ", repr(o))
         error("This object should be implemented by the child class.")
 
     def abs(self, o, *operands):
-        print "\n\nVisiting Abs: ", repr(o)
+        print("\n\nVisiting Abs: ", repr(o))
         error("This object should be implemented by the child class.")
 
     # -------------------------------------------------------------------------
@@ -401,7 +389,7 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting Grad: " + repr(o))
 
         # Get expression
-        derivative_expr, = o.operands()
+        derivative_expr, = o.ufl_operands
 
         # Get components
         components = self.component()
@@ -566,7 +554,7 @@ class QuadratureTransformerBase(Transformer):
 
         # Get indexed expression and index, map index to current value
         # and update components
-        indexed_expr, index = o.operands()
+        indexed_expr, index = o.ufl_operands
         self._components.push(self.visit(index))
 
         # Visit expression subtrees and generate code.
@@ -600,7 +588,7 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting IndexSum: " + str(tree_format(o)))
 
         # Get expression and index that we're summing over
-        summand, multiindex = o.operands()
+        summand, multiindex = o.ufl_operands
         index, = multiindex
 
         # Loop index range, update index/value dict and generate code
@@ -706,7 +694,7 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting PositiveRestricted: " + repr(o))
 
         # Just get the first operand, there should only be one.
-        restricted_expr = o.operands()
+        restricted_expr = o.ufl_operands
         ffc_assert(len(restricted_expr) == 1, "Only expected one operand for restriction: " + repr(restricted_expr))
         ffc_assert(self.restriction is None, "Expression is restricted twice: " + repr(restricted_expr))
 
@@ -721,7 +709,7 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting NegativeRestricted: " + repr(o))
 
         # Just get the first operand, there should only be one.
-        restricted_expr = o.operands()
+        restricted_expr = o.ufl_operands
         ffc_assert(len(restricted_expr) == 1, "Only expected one operand for restriction: " + repr(restricted_expr))
         ffc_assert(self.restriction is None, "Expression is restricted twice: " + repr(restricted_expr))
 
@@ -736,7 +724,7 @@ class QuadratureTransformerBase(Transformer):
         ffc_assert(self.avg is None, "Not expecting nested averages.")
 
         # Just get the first operand, there should only be one.
-        expr, = o.operands()
+        expr, = o.ufl_operands
 
         # Set average marker, visit operand and reset marker
         self.avg = "cell"
@@ -750,7 +738,7 @@ class QuadratureTransformerBase(Transformer):
         ffc_assert(self.entity_type != "cell", "Cannot take facet_avg in a cell integral.")
 
         # Just get the first operand, there should only be one.
-        expr, = o.operands()
+        expr, = o.ufl_operands
 
         # Set average marker, visit operand and reset marker
         self.avg = "facet"
@@ -766,7 +754,7 @@ class QuadratureTransformerBase(Transformer):
         #print("\n\nVisiting ComponentTensor:\n" + str(tree_format(o)))
 
         # Get expression and indices
-        component_expr, indices = o.operands()
+        component_expr, indices = o.ufl_operands
 
         # Get current component(s)
         components = self.component()
@@ -776,7 +764,7 @@ class QuadratureTransformerBase(Transformer):
 
         # Update the index dict (map index values of current known indices to
         # those of the component tensor)
-        for i, v in izip(indices._indices, components):
+        for i, v in zip(indices._indices, components):
             self._index2value.push(i, v)
 
         # Push an empty component tuple
@@ -804,7 +792,7 @@ class QuadratureTransformerBase(Transformer):
         c0, c1 = component[0], component[1:]
 
         # Get first operand
-        op = o.operands()[c0]
+        op = o.ufl_operands[c0]
 
         # Evaluate subtensor with this subcomponent
         self._components.push(c1)
@@ -830,17 +818,16 @@ class QuadratureTransformerBase(Transformer):
         # Set domain type
         self.integral_type = integral_type
 
-        #print integrand
-        #print tree_format(integrand, 0, False)
-        # Get terms.
+        # Get terms
         terms = self.visit(integrand)
 
+        # Get formatting
         f_nzc = format["nonzero columns"](0).split("0")[0]
 
         # Loop code and add weight and scale factor to value and sort after
         # loop ranges.
         new_terms = {}
-        for key, val in terms.items():
+        for key, val in sorted(terms.items()):
             # If value was zero continue.
             if val is None:
                 continue
@@ -964,16 +951,15 @@ class QuadratureTransformerBase(Transformer):
         ffc_element = create_element(ufl_element)
 
         # Assuming that mappings for all basisfunctions are equal
-        # (they should be).
         ffc_sub_element = create_element(local_elem)
         transformation = ffc_sub_element.mapping()[0]
+        ffc_assert(all(transformation == mapping for mapping in ffc_sub_element.mapping()),
+                   "Assuming subelement mappings are equal but they differ.")
 
         # Generate FFC multi index for derivatives.
         tdim = self.tdim # FIXME: ufl_element.domain().topological_dimension() ???
-        multiindices = FFCMultiIndex([range(tdim)]*len(derivatives)).indices
+        multiindices = FFCMultiIndex([list(range(tdim))]*len(derivatives)).indices
 
-        #print "in create_auxiliary"
-        #print "component = ", component
         return (component, local_elem, local_comp, local_offset, ffc_element, transformation, multiindices)
 
     def _get_current_entity(self):
@@ -1018,14 +1004,14 @@ class QuadratureTransformerBase(Transformer):
         if self.restriction in ("+", "-"):
             space_dim *= 2
 
-        # If we have a restricted function and domain type is custom,
-        # then offset also the basis function access
-        if self.restriction in ("+", "-") and self.integral_type == "custom" and offset != "":
-            loop_index = format["add"]([loop_index, offset])
-
         # Create basis access, we never need to map the entry in the basis table
         # since we will either loop the entire space dimension or the non-zeros.
-        basis_access = format["component"]("", [f_ip, loop_index])
+        if self.restriction in ("+", "-") and self.integral_type == "custom" and offset != "":
+            # Special case access for custom integrals (all basis functions stored in flattened array)
+            basis_access = format["component"]("", [f_ip, format["add"]([loop_index, offset])])
+        else:
+            # Normal basis function access
+            basis_access = format["component"]("", [f_ip, loop_index])
 
         # Get current cell entity, with current restriction considered
         entity = self._get_current_entity()
@@ -1033,6 +1019,11 @@ class QuadratureTransformerBase(Transformer):
         name, non_zeros, zeros, ones = self.name_map[name]
         loop_index_range = shape(self.unique_tables[name])[1]
 
+        # If domain type is custom, then special-case set loop index
+        # range since table is empty
+        if self.integral_type == "custom":
+            loop_index_range = ffc_element.space_dimension() # different from `space_dimension`...
+
         basis = ""
         # Ignore zeros if applicable
         if zeros and (self.optimise_parameters["ignore zero tables"] or self.optimise_parameters["remove zero terms"]):
@@ -1097,6 +1088,13 @@ class QuadratureTransformerBase(Transformer):
 
         # Get the index range of the loop index.
         loop_index_range = shape(self.unique_tables[psi_name])[1]
+
+        # If domain type is custom, then special-case set loop index
+        # range since table is empty
+        if self.integral_type == "custom":
+            loop_index_range = ffc_element.space_dimension()
+
+        # Create loop index
         if loop_index_range > 1:
             # Pick first free index of secondary type
             # (could use primary indices, but it's better to avoid confusion).
