@@ -18,7 +18,7 @@
 # along with FFC. If not, see <http://www.gnu.org/licenses/>.
 #
 # First added:  2007-03-16
-# Last changed: 2014-04-23
+# Last changed: 2014-06-09
 #
 # Hacked by Marie E. Rognes 2013
 # Modified by Anders Logg 2014
@@ -106,15 +106,16 @@ def flatten_psi_tables(tables, entity_type):
 
                         # Iterate over the innermost tables for each scalar component
                         for component, psi_table in component_tables:
+
                             # Generate the table name.
                             name = generate_psi_name(counter, entity_type, entity, component, derivs, avg)
 
                             # Verify shape of basis (can be omitted for speed if needed).
-                            ffc_assert(num_points is None or (len(numpy.shape(psi_table)) == 2 and numpy.shape(psi_table)[0] == num_points),
-                                       "This table has the wrong shape: " + str(psi_table))
+                            #if not (num_points is None or (len(numpy.shape(psi_table)) == 2 and numpy.shape(psi_table)[0] == num_points)):
+                            #    error("This table has the wrong shape: " + str(psi_table))
                             # Verify uniqueness of names
-                            ffc_assert(name not in flat_tables,
-                                        "Table name is not unique, something is wrong:\n  name = %s\n  table = %s\n" % (name, flat_tables))
+                            if name in flat_tables:
+                                error("Table name is not unique, something is wrong:\n  name = %s\n  table = %s\n" % (name, flat_tables))
 
                             # Store table with unique name
                             flat_tables[name] = psi_table
