@@ -12,7 +12,7 @@ function "foo", one should only need to use the data stored
 in the intermediate representation under the key "foo".
 """
 
-# Copyright (C) 2009-2013 Anders Logg
+# Copyright (C) 2009-2014 Anders Logg
 #
 # This file is part of FFC.
 #
@@ -31,10 +31,7 @@ in the intermediate representation under the key "foo".
 #
 # Modified by Marie E. Rognes 2010
 # Modified by Kristian B. Oelgaard 2010
-# Modified by Martin Alnaes, 2013
-#
-# First added:  2009-12-16
-# Last changed: 2014-04-24
+# Modified by Martin Alnaes, 2013-2014
 
 # Python modules
 from itertools import chain
@@ -42,12 +39,12 @@ from itertools import chain
 # Import UFL
 import ufl
 from ufl.classes import Measure
-from ufl.geometry import cellname2dim
+from ufl.cell import cellname2dim
 
 # FFC modules
 from ffc.utils import compute_permutations, product
 from ffc.log import info, error, begin, end, debug_ir, ffc_assert, warning
-from ffc.fiatinterface import create_element, cellname_to_num_entities, reference_cell
+from ffc.fiatinterface import create_element, reference_cell
 from ffc.mixedelement import MixedElement
 from ffc.enrichedelement import EnrichedElement, SpaceOfReals
 from ffc.quadratureelement import QuadratureElement
@@ -93,7 +90,7 @@ def compute_ir(analysis, parameters):
 
     # Compute and flatten representation of integrals
     info("Computing representation of integrals")
-    irs = [_compute_integral_ir(fd, i, parameters) \
+    irs = [_compute_integral_ir(fd, i, element_numbers, parameters) \
                for (i, fd) in enumerate(form_datas)]
     ir_integrals = [ir for ir in chain(*irs) if not ir is None]
 
@@ -141,11 +138,12 @@ def _compute_dofmap_ir(ufl_element, element_id, element_numbers):
     # Create FIAT element
     element = create_element(ufl_element)
     domain, = ufl_element.domains() # Assuming single domain
-    cellname = domain.cell().cellname()
+    cell = domain.cell()
+    cellname = cell.cellname()
 
     # Precompute repeatedly used items
     num_dofs_per_entity = _num_dofs_per_entity(element)
-    facet_dofs = _tabulate_facet_dofs(element, cellname)
+    facet_dofs = _tabulate_facet_dofs(element, cell)
 
     # Store id
     ir = {"id": element_id}
@@ -159,7 +157,7 @@ def _compute_dofmap_ir(ufl_element, element_id, element_numbers):
     ir["local_dimension"] = element.space_dimension()
     ir["num_facet_dofs"] = len(facet_dofs[0])
     ir["num_entity_dofs"] = num_dofs_per_entity
-    ir["tabulate_dofs"] = _tabulate_dofs(element, cellname)
+    ir["tabulate_dofs"] = _tabulate_dofs(element, cell)
     ir["tabulate_facet_dofs"] = facet_dofs
     ir["tabulate_entity_dofs"] = (element.entity_dofs(), num_dofs_per_entity)
     ir["tabulate_coordinates"] = _tabulate_coordinates(ufl_element, element)
@@ -200,7 +198,7 @@ def _needs_mesh_entities(element):
     else:
         return [d > 0 for d in num_dofs_per_entity]
 
-def _compute_integral_ir(form_data, form_id, parameters):
+def _compute_integral_ir(form_data, form_id, element_numbers, parameters):
     "Compute intermediate represention for form integrals."
 
     irs = []
@@ -216,6 +214,7 @@ def _compute_integral_ir(form_data, form_id, parameters):
         ir = r.compute_integral_ir(itg_data,
                                    form_data,
                                    form_id,
+                                   element_numbers,
                                    parameters)
 
         # Append representation
@@ -234,36 +233,21 @@ def _compute_form_ir(form_data, form_id, element_numbers):
     ir["members"] = not_implemented
     ir["constructor"] = not_implemented
     ir["destructor"] = not_implemented
-    ir["signature"] = form_data.signature
-    ir["rank"] = form_data.rank
-    ir["num_coefficients"] = form_data.num_coefficients
+    ir["signature"] = form_data.original_form.signature()
 
-    ir["num_cell_domains"] = form_data.num_sub_domains.get("cell",0)
-    ir["num_exterior_facet_domains"] = form_data.num_sub_domains.get("exterior_facet",0)
-    ir["num_interior_facet_domains"] = form_data.num_sub_domains.get("interior_facet",0)
-    ir["num_point_domains"] = form_data.num_sub_domains.get("point",0)
-    ir["num_custom_domains"] = form_data.num_sub_domains.get("custom",0)
-
-    ir["has_cell_integrals"] = _has_foo_integrals("cell", form_data)
-    ir["has_exterior_facet_integrals"] = _has_foo_integrals("exterior_facet", form_data)
-    ir["has_interior_facet_integrals"] = _has_foo_integrals("interior_facet", form_data)
-    ir["has_point_integrals"] = _has_foo_integrals("point", form_data)
-    ir["has_custom_integrals"] = _has_foo_integrals("custom", form_data)
+    ir["rank"] = len(form_data.original_form.arguments())
+    ir["num_coefficients"] = len(form_data.reduced_coefficients)
+    ir["original_coefficient_positions"] = form_data.original_coefficient_positions
 
     ir["create_finite_element"] = [element_numbers[e] for e in form_data.elements]
     ir["create_dofmap"] = [element_numbers[e] for e in form_data.elements]
 
-    ir["create_cell_integral"] = _create_foo_integral("cell", form_data)
-    ir["create_exterior_facet_integral"] = _create_foo_integral("exterior_facet", form_data)
-    ir["create_interior_facet_integral"] = _create_foo_integral("interior_facet", form_data)
-    ir["create_point_integral"] = _create_foo_integral("point", form_data)
-    ir["create_custom_integral"] = _create_foo_integral("custom", form_data)
-
-    ir["create_default_cell_integral"] = _create_default_foo_integral("cell", form_data)
-    ir["create_default_exterior_facet_integral"] = _create_default_foo_integral("exterior_facet", form_data)
-    ir["create_default_interior_facet_integral"] = _create_default_foo_integral("interior_facet", form_data)
-    ir["create_default_point_integral"] = _create_default_foo_integral("point", form_data)
-    ir["create_default_custom_integral"] = _create_default_foo_integral("custom", form_data)
+    integral_types = ["cell", "exterior_facet", "interior_facet", "point", "custom"]
+    for integral_type in integral_types:
+        ir["num_%s_domains" % integral_type] = _num_foo_domains(integral_type, form_data)
+        ir["has_%s_integrals" % integral_type] = _has_foo_integrals(integral_type, form_data)
+        ir["create_%s_integral" % integral_type] = _create_foo_integral(integral_type, form_data)
+        ir["create_default_%s_integral" % integral_type] = _create_default_foo_integral(integral_type, form_data)
 
     return ir
 
@@ -322,8 +306,7 @@ def _generate_physical_offsets(ufl_element, offset=0):
         element = create_element(ufl_element)
         offsets = [offset]*element.space_dimension()
     else:
-        raise NotImplementedError, \
-            "This element combination is not implemented"
+        raise NotImplementedError("This element combination is not implemented")
     return offsets
 
 def _evaluate_dof(ufl_element, element):
@@ -449,17 +432,17 @@ def _tabulate_coordinates(ufl_element, element):
     data = {}
     data["tdim"] = domain.topological_dimension()
     data["gdim"] = domain.geometric_dimension()
-    data["points"] = [L.pt_dict.keys()[0] for L in element.dual_basis()]
+    data["points"] = [sorted(L.pt_dict.keys())[0] for L in element.dual_basis()]
     return data
 
-def _tabulate_dofs(element, cellname):
+def _tabulate_dofs(element, cell):
     "Compute intermediate representation of tabulate_dofs."
 
     if isinstance(element, SpaceOfReals):
         return None
 
     # Extract number of entities for each dimension for this cell
-    num_entities = cellname_to_num_entities[cellname]
+    num_entities = cell.num_entities()
 
     # Extract number of dofs per entity for each element
     elements = all_elements(element)
@@ -484,17 +467,17 @@ def _tabulate_dofs(element, cellname):
 
     return (dofs_per_element, num_dofs_per_element, num_entities, need_offset, fakes)
 
-def _tabulate_facet_dofs(element, cellname):
+def _tabulate_facet_dofs(element, cell):
     "Compute intermediate representation of tabulate_facet_dofs."
 
     # Compute incidences
-    incidence = __compute_incidence(cellname2dim[cellname])
+    incidence = __compute_incidence(cell.topological_dimension())
 
     # Get topological dimension
     D = max([pair[0][0] for pair in incidence])
 
     # Get the number of facets
-    num_facets = cellname_to_num_entities[cellname][-2]
+    num_facets = cell.num_facets()
 
     # Find out which entities are incident to each facet
     incident = num_facets*[None]
@@ -543,7 +526,7 @@ def _interpolate_vertex_values(ufl_element, element):
     vertices = fiat_cell.get_vertices()
 
     # Compute data for each constituent element
-    extract = lambda values: values[values.keys()[0]].transpose()
+    extract = lambda values: values[sorted(values.keys())[0]].transpose()
     all_fiat_elm = all_elements(element)
     ir["element_data"] = [{
                            # See note in _evaluate_dofs
@@ -576,6 +559,10 @@ def _create_foo_integral(integral_type, form_data):
     return [itg_data.subdomain_id for itg_data in form_data.integral_data
            if itg_data.integral_type == integral_type and isinstance(itg_data.subdomain_id, int)]
 
+def _num_foo_domains(integral_type, form_data):
+    "Compute intermediate representation of num_foo_domains."
+    return form_data.num_sub_domains.get(integral_type, 0)
+
 def _has_foo_integrals(integral_type, form_data):
     "Compute intermediate representation of has_foo_integrals."
     v = (form_data.num_sub_domains.get(integral_type,0) > 0
@@ -647,7 +634,7 @@ def __compute_sub_simplices(D, d):
     if d == 0:
         return [[i] for i in range(num_vertices)]
     elif d == D:
-        return [range(num_vertices)]
+        return [list(range(num_vertices))]
 
     # Compute all permutations of num_vertices - (d + 1)
     permutations = compute_permutations(num_vertices - d - 1, num_vertices)
