@@ -18,18 +18,18 @@
 # First added:  2010-01-29
 # Last changed: 2013-01-31
 
-from cppcode import evaluate_basis_code_fiat
+from .cppcode import evaluate_basis_code_fiat
 from ufl import FiniteElement
 from ffc.fiatinterface import create_element, reference_cell
 from ffc.mixedelement import MixedElement as FFCMixedElement
 from ffc.log import info, error, debug
 import numpy
 import sys, os, numpy, shutil
-from test_common import compile_element, print_results, compile_gcc_code,\
+from .test_common import compile_element, print_results, compile_gcc_code,\
                         run_code, get_element_name, verify_values
 import time
 from ffc.log import push_level, pop_level, CRITICAL, INFO
-from elements import single_elements, mixed_elements
+from .elements import single_elements, mixed_elements
 
 # Some random points
 random_points = {1: [(0.114,), (0.349,), (0.986,)],
@@ -63,7 +63,7 @@ def get_data(ufl_element):
     ref_coords = reference_cell(domain.cell().cellname()).get_vertices()
 
     # Get the locations of the fiat element dofs.
-    elem_points =  [L.pt_dict.keys()[0] for L in element.dual_basis()]
+    elem_points =  [list(L.pt_dict.keys())[0] for L in element.dual_basis()]
 
     # Add some random points.
     geo_dim = domain.geometric_dimension()
@@ -145,7 +145,7 @@ def get_fiat_values(ufl_element):
                             deriv_vals = values[deriv]
                             row[i].append(deriv_vals[i][c][p])
                 else:
-                    print values
+                    print(values)
                     error("Did not expect tensor elements")
             new_row = []
             for r in row:
