@@ -1,13 +1,13 @@
-// This is UFC (Unified Form-assembly Code) v. 1.4.0.
+// This is UFC (Unified Form-assembly Code) v. 1.5.0.
 // This code is released into the public domain.
 //
-// The FEniCS Project (http://www.fenicsproject.org/) 2006-2014.
+// The FEniCS Project (http://www.fenicsproject.org/) 2006-2015.
 
 #ifndef __UFC_H
 #define __UFC_H
 
 #define UFC_VERSION_MAJOR 1
-#define UFC_VERSION_MINOR 4
+#define UFC_VERSION_MINOR 5
 #define UFC_VERSION_MAINTENANCE 0
 
 #include <vector>
@@ -16,7 +16,7 @@
 
 #include <ufc_geometry.h>
 
-const char UFC_VERSION[] = "1.4.0";
+const char UFC_VERSION[] = "1.5.0";
 
 namespace ufc
 {
@@ -244,7 +244,7 @@ namespace ufc
     /// Return the number of dofs on each cell facet
     virtual std::size_t num_facet_dofs() const = 0;
 
-    /// Return the number of dofs associated with each cell entity of
+   /// Return the number of dofs associated with each cell entity of
     /// dimension d
     virtual std::size_t num_entity_dofs(std::size_t d) const = 0;
 
@@ -262,7 +262,7 @@ namespace ufc
                                       std::size_t d, std::size_t i) const = 0;
 
     /// Tabulate the coordinates of all dofs on a cell
-    virtual void tabulate_coordinates(double** dof_coordinates,
+    virtual void tabulate_coordinates(double* dof_coordinates,
                                       const double* vertex_coordinates) const = 0;
 
     /// Return the number of sub dofmaps (for a mixed element)
@@ -394,13 +394,14 @@ namespace ufc
     /// Return the number of cells involved in evaluation of the integral
     virtual std::size_t num_cells() const = 0;
 
-    /// Tabulate the tensor for the contribution from custom domain
+    /// Tabulate the tensor for the contribution from a custom domain
     virtual void tabulate_tensor(double* A,
                                  const double * const * w,
                                  const double* vertex_coordinates,
                                  std::size_t num_quadrature_points,
                                  const double* quadrature_points,
                                  const double* quadrature_weights,
+                                 const double* facet_normals,
                                  int cell_orientation) const = 0;
 
   };
@@ -430,6 +431,9 @@ namespace ufc
     /// Return a string identifying the form
     virtual const char* signature() const = 0;
 
+    /// Return original coefficient position for each coefficient (0 <= i < n)
+    virtual std::size_t original_coefficient_position(std::size_t i) const = 0;
+
     /// Return the rank of the global tensor (r)
     virtual std::size_t rank() const = 0;
 
